package com.perforce.p4simulink;

/**
 * Copyright (C) 2014 Perforce Software. All rights reserved.
 *
 * Please see LICENSE.txt in top-level folder of this distribution.
 */

import java.io.File;
import java.nio.file.Path;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.mathworks.cmlink.api.ApplicationInteractor;
import com.mathworks.cmlink.api.ConfigurationManagementException;
import com.mathworks.cmlink.api.version.r14a.CMAdapter;
import com.mathworks.cmlink.api.version.r14a.CMAdapterFactory;
import com.mathworks.cmlink.api.version.r14a.CMRepository;
import com.mathworks.cmlink.util.interactor.NullApplicationInteractor;
import com.perforce.p4simulink.connection.Config;

public class P4AdapterFactory implements CMAdapterFactory {

	private static final Logger log = LogManager
			.getLogger(P4AdapterFactory.class.getName());
	private Config config;

	@Override
	public boolean isDirSandboxForThisAdapter(File file) {
		if (file == null) {
			return false;
		}

		// Walk up through the file tree looking for a Perforce config file. If
		// we find one,
		// then we assume that the sandbox is under Perforce control. If we make
		// it to the root of the filesystem
		// and haven't found a config file, then we'll just return false and
		// assume that it's currently outside
		// Perforce control.
		Path path = file.toPath();
		while (path != null) {
			File configFile = path.toFile();
			if (configFile.isDirectory()) {
				configFile = new File(configFile.getAbsolutePath()
						+ File.separator + ".p4config");
			}
			if (configFile.exists()) {
				return true;
			} else {
				path = path.getParent();
			}
		}
		return false;
	}

	public CMRepository getRepository() {
		return getRepository(new NullApplicationInteractor());
	}

	public CMAdapter getAdapterForThisSandboxDir(File file)
			throws ConfigurationManagementException {
		return getAdapterForThisSandboxDir(file,
				new NullApplicationInteractor());
	}

	public CMAdapter getAdapterForThisSandboxDir(File file,
			ApplicationInteractor applicationInteractor)
			throws ConfigurationManagementException {
		log.debug("Constructing new P4Adapter");
		return getAdapterForThisSandboxDir(file, applicationInteractor,
				new Config(file));
	}

	public CMAdapter getAdapterForThisSandboxDir(File file,
			ApplicationInteractor applicationInteractor, Config config)
			throws ConfigurationManagementException {
		log.debug("Constructing new P4Adapter");
		this.config = config;
		if (config != null) {
			config.load(false);
		}
		P4Adapter adapter = new P4Adapter(applicationInteractor, config);
		adapter.setSandboxRoot(file);

		return adapter;
	}

	public CMRepository getRepository(
			ApplicationInteractor applicationInteractor) {
		log.debug("Constructing new P4Repository");
		CMRepository repository = null;
		try {
			repository = new P4Repository(applicationInteractor);
		} catch (P4CMException e) {
			log.info("Error constructing a new P4Repository.");
		}
		return repository;
	}

	@Override
	public String getName() {
		return "P4Simulink";
	}

	public String getDescription() {
		String description = "Perforce SCM Integration for Simulink";

		if (config == null) {
			config = new Config();
		}

		try {
			// force reload
			config.load(true);
		} catch (P4CMException e) {
		}

		if (config.getP4JavaURI() == null
				|| config.getP4JavaURI().length() == 0) {
			description += "\nNo configuration file(s) found. Please create a file under "
					+ config.getConfigFile().getAbsolutePath();
		}
		return description;
	}
}
